# coding: utf-8

hestiacp_ruby_gem_version = "3.3.0"
Gem.paths = {
  "GEM_HOME" => "#{__dir__}/vendor/bundle/ruby/#{hestiacp_ruby_gem_version}",
  "GEM_PATH" => "#{__dir__}/vendor/bundle/ruby/#{hestiacp_ruby_gem_version}",
}

require "rubygems"
require "sinatra"
require "puma"

require_relative "classes/config"
require_relative "classes/gitinfo"
require_relative "classes/db"
require_relative "classes/systeminfo"

def print_error_page(error_status, error_meaasge)
  @page_name = "Ошибка выполнения"
  @status_err = error_status
  @error_msg = error_meaasge
  halt erb(:page5xx)
end

def sanitize_filename(filename)
  filename = filename.strip
  sanitized = filename.gsub(/[^a-zA-Z0-9_]/, "_")
  sanitized.gsub!(/_+/, "_")
  sanitized.gsub!(/^_+|_+$/, "")
  sanitized
end

cfg = IniConfig.new()
db = DBase.new()

set :port, cfg.get_port
set :public_folder, File.dirname(__FILE__) + "/public"
use Rack::Session::Pool, :expire_after => 2592000

get "/" do
  @page_name = "Система сборки rpm пакетов"
  repo = GitRepo.new(cfg.get_repo, db)
  if repo.path.nil?
    print_error_page(503, "Путь к репозиториям не существует")
  else
    repo_data = repo.getrepos
    @repos_number = repo_data.length
    @rcp_number = db.get_recips.length
    erb :index
  end
end

get "/repos/:reponame/:branch" do
  @page_name = params["reponame"]
  repo = GitRepo.new(cfg.get_repo, db)
  if repo.path.nil?
    print_error_page(503, "Путь к репозиториям не существует")
  else
    tmp_result = repo.repo_info(params["reponame"], params["branch"])
    if tmp_result[:error].nil?
      @repo_data = tmp_result
      @ip_addr = systeminfo_get_ip
      @user_name = systeminfo_get_username
      @path_to_repo = File.join(repo.path, params["reponame"] + ".git")
      erb :gitinfo
    else
      print_error_page(503, tmp_result[:error])
    end
  end
end

get "/repos/:reponame" do
  @page_name = params["reponame"]
  repo = GitRepo.new(cfg.get_repo, db)
  if repo.path.nil?
    print_error_page(503, "Путь к репозиториям не существует")
  else
    tmp_result = repo.repo_info(params["reponame"])
    if tmp_result[:error].nil?
      @repo_data = tmp_result
      @ip_addr = systeminfo_get_ip
      @user_name = systeminfo_get_username
      @path_to_repo = File.join(repo.path, params["reponame"] + ".git")
      erb :gitinfo
    else
      print_error_page(503, tmp_result[:error])
    end
  end
end

post "/gitdelete/:reponame" do
  input_name = params["reponamedup"]
  repo = GitRepo.new(cfg.get_repo, db)
  if repo.path.nil?
    print_error_page(503, "Путь к репозиториям не существует")
  else
    if params["reponame"] == input_name
      repo.delete_repo(input_name)
    end
    redirect "/repos"
  end
end

get "/gitdelete" do
  redirect "/repos"
end

get "/repos" do
  @page_name = "Список доступных git репозиториев"
  repo = GitRepo.new(cfg.get_repo, db)
  if repo.path.nil?
    print_error_page(503, "Путь к репозиториям не существует")
  else
    @repo_data = repo.getrepos
    erb :gits
  end
end

get "/gitcreate" do
  @page_name = "Создать новый пустой git репозиторий"
  @error_data = session[:gitcreate_error]
  session[:gitcreate_error] = nil
  erb :newgit
end

post "/gitcreate" do
  if request.form_data?
    if params["gitname"].nil?
      session[:gitcreate_error] = "Не задано имя репозитория"
    else
      filename = sanitize_filename(params["gitname"])
      if filename != ""
        repo = GitRepo.new(cfg.get_repo, db)
        if repo.path.nil?
          print_error_page(503, "Путь к репозиториям не существует")
        else
          if repo.create_git(filename, params["gitdescr"]) != 0
            session[:gitcreate_error] = repo.error
          else
            redirect "/repos"
            return
          end
        end
      else
        session[:gitcreate_error] = "Название репозитория должно содержать только символы a-z, A-Z. 0-9 и _"
      end
    end
  end
  redirect "/gitcreate"
end

get "/recips" do
  @page_name = "Список доступных сценариев сборки"
  @rcp_data = db.get_recips
  erb :receips
end

not_found do
  status 404
  @page_name = "Кто-то потерялся"
  @req_referer = request.referer
  erb :page404
end
